@@ -70,9 +70,6 @@ struct _EBookBackendEwsPrivate {
 	gchar *oab_url;
 	gchar *folder_name;
 
-	gchar *username;
-	gchar *password;
-
 	EwsBookBackendSqliteDB *ebsdb;
 
 	gboolean only_if_exists;
@@ -167,7 +164,7 @@ convert_error_to_edb_error (GError **perror)
 	if ((*perror)->domain == EWS_CONNECTION_ERROR) {
 		switch ((*perror)->code) {
 		case EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED:
-			error = EDB_ERROR (AUTHENTICATION_FAILED);
+			error = EDB_ERROR_EX (AUTHENTICATION_FAILED, (*perror)->message);
 			break;
 		case EWS_CONNECTION_ERROR_CANCELLED:
 			break;
@@ -175,11 +172,11 @@ convert_error_to_edb_error (GError **perror)
 		case EWS_CONNECTION_ERROR_MANAGEDFOLDERNOTFOUND:
 		case EWS_CONNECTION_ERROR_PARENTFOLDERNOTFOUND:
 		case EWS_CONNECTION_ERROR_PUBLICFOLDERSERVERNOTFOUND:
-			error = EDB_ERROR (NO_SUCH_BOOK);
+			error = EDB_ERROR_EX (NO_SUCH_BOOK, (*perror)->message);
 			break;
 		case EWS_CONNECTION_ERROR_EVENTNOTFOUND:
 		case EWS_CONNECTION_ERROR_ITEMNOTFOUND:
-			error = EDB_ERROR (CONTACT_NOT_FOUND);
+			error = EDB_ERROR_EX (CONTACT_NOT_FOUND, (*perror)->message);
 			break;
 		}
 	}
@@ -450,10 +447,10 @@ ebews_set_full_name (ESoapMessage *msg,
 		return;
 
 	if (name->given)
-		e_ews_message_write_string_parameter(msg, "GivenName", NULL, name->given);
+		e_ews_message_write_string_parameter (msg, "GivenName", NULL, name->given);
 
 	if (name->additional && *name->additional)
-		e_ews_message_write_string_parameter(msg, "MiddleName", NULL, name->additional);
+		e_ews_message_write_string_parameter (msg, "MiddleName", NULL, name->additional);
 
 	e_contact_name_free (name);
 }
@@ -471,10 +468,11 @@ ebews_set_birth_date (ESoapMessage *message,
 	if (!date)
 		return;
 
-	birthday = g_strdup_printf("%04d-%02d-%02dT00:00:00",
-			      date->year, date->month, date->day);
+	birthday = g_strdup_printf (
+		"%04d-%02d-%02dT00:00:00",
+		date->year, date->month, date->day);
 
-	e_ews_message_write_string_parameter(message, "Birthday", NULL, birthday);
+	e_ews_message_write_string_parameter (message, "Birthday", NULL, birthday);
 
 	g_free (birthday);
 
@@ -502,7 +500,7 @@ add_entry (ESoapMessage *msg,
 		if (include_hdr)
 			e_soap_message_start_element (msg, include_hdr, NULL, NULL);
 
-		e_ews_message_write_string_parameter_with_attribute(msg, "Entry", NULL, entry_val, "Key", entry_name);
+		e_ews_message_write_string_parameter_with_attribute (msg, "Entry", NULL, entry_val, "Key", entry_name);
 
 		g_free (entry_val);
 		return TRUE;
@@ -622,14 +620,14 @@ convert_indexed_contact_property_to_updatexml (ESoapMessage *message,
 {
 	gboolean delete_field = FALSE;
 
-	if(!value || !g_strcmp0(value, ""))
+	if (!value || !g_strcmp0 (value, ""))
 		delete_field = TRUE;
 	e_ews_message_start_set_indexed_item_field (message, name , prefix, "Contact", key, delete_field);
 
 	if (!delete_field)
 	{
 		e_soap_message_start_element (message, element_name, NULL, NULL);
-		e_ews_message_write_string_parameter_with_attribute(message, "Entry", NULL, value, "Key", key);
+		e_ews_message_write_string_parameter_with_attribute (message, "Entry", NULL, value, "Key", key);
 		e_soap_message_end_element (message);
 	}
 	e_ews_message_end_set_indexed_item_field (message, delete_field);
@@ -648,10 +646,10 @@ ebews_set_full_name_changes (ESoapMessage *message,
 		return;
 
 	if (g_ascii_strcasecmp (name->given, old_name->given))
-		convert_contact_property_to_updatexml(message, "GivenName", name->given, "contacts", NULL, NULL);
+		convert_contact_property_to_updatexml (message, "GivenName", name->given, "contacts", NULL, NULL);
 
 	if (g_ascii_strcasecmp (name->additional, old_name->additional))
-		convert_contact_property_to_updatexml(message, "MiddleName", name->additional, "contacts", NULL, NULL);
+		convert_contact_property_to_updatexml (message, "MiddleName", name->additional, "contacts", NULL, NULL);
 
 	e_contact_name_free (name);
 	e_contact_name_free (old_name);
@@ -672,10 +670,11 @@ ebews_set_birth_date_changes (ESoapMessage *message,
 	if (e_contact_date_equal (new_date, old_date))
 		return;
 
-	birthday = g_strdup_printf("%04d-%02d-%02dT00:00:00",
-			      new_date->year, new_date->month, new_date->day);
+	birthday = g_strdup_printf (
+		"%04d-%02d-%02dT00:00:00",
+		new_date->year, new_date->month, new_date->day);
 
-	convert_contact_property_to_updatexml(message, "Birthday", birthday, "contacts", NULL, NULL);
+	convert_contact_property_to_updatexml (message, "Birthday", birthday, "contacts", NULL, NULL);
 	g_free (birthday);
 }
 
@@ -720,7 +719,7 @@ convert_indexed_contact_property_to_updatexml_physical_address (ESoapMessage *me
 	gchar * fielduri = NULL;
 	gboolean delete_field = FALSE;
 
-	if(!value || !g_strcmp0(value, ""))
+	if (!value || !g_strcmp0 (value, ""))
 		delete_field = TRUE;
 
 	fielduri = g_strconcat (name, ":", uri_element, NULL);
@@ -784,9 +783,9 @@ ebews_set_address_changes (ESoapMessage *message,
                            EContact *new,
                            EContact *old)
 {
-	compare_address(message, new, old, E_CONTACT_ADDRESS_WORK, "Business");
-	compare_address(message, new, old, E_CONTACT_ADDRESS_HOME, "Home");
-	compare_address(message, new, old, E_CONTACT_ADDRESS_OTHER, "Other");
+	compare_address (message, new, old, E_CONTACT_ADDRESS_WORK, "Business");
+	compare_address (message, new, old, E_CONTACT_ADDRESS_HOME, "Home");
+	compare_address (message, new, old, E_CONTACT_ADDRESS_OTHER, "Other");
 }
 
 static void
@@ -887,7 +886,7 @@ convert_contact_to_xml (ESoapMessage *msg,
 	gint i, element_type;
 
 	/* Prepare Contact node in the SOAP message */
-	e_soap_message_start_element(msg, "Contact", NULL, NULL);
+	e_soap_message_start_element (msg, "Contact", NULL, NULL);
 
 	for (i = 0; i < G_N_ELEMENTS (mappings); i++) {
 		element_type = mappings[i].element_type;
@@ -950,7 +949,7 @@ ews_create_contact_cb (GObject *object,
 	}
 
 	if (error) {
-		g_warning("Error while Creating contact: %s", error->message);
+		g_warning ("Error while Creating contact: %s", error->message);
 		e_data_book_respond_create_contacts (create_contact->book, create_contact->opid, EDB_ERROR_EX (OTHER_ERROR, error->message), NULL);
 	}
 
@@ -974,10 +973,11 @@ e_book_backend_ews_create_contacts (EBookBackend *backend,
 	EBookBackendEwsPrivate *priv;
 
 	if (vcards->next != NULL) {
-		e_data_book_respond_create_contacts (book, opid,
-						     EDB_ERROR_EX (NOT_SUPPORTED,
-						     _("The backend does not support bulk additions")),
-						     NULL);
+		e_data_book_respond_create_contacts (
+			book, opid,
+			EDB_ERROR_EX (NOT_SUPPORTED,
+			_("The backend does not support bulk additions")),
+			NULL);
 		return;
 	}
 
@@ -1183,7 +1183,7 @@ ews_modify_contact_cb (GObject *object,
 	}
 
 	if (error) {
-		g_warning("Error while Modifying contact: %s", error->message);
+		g_warning ("Error while Modifying contact: %s", error->message);
 
 		e_data_book_respond_modify_contacts (modify_contact->book, modify_contact->opid, EDB_ERROR_EX (OTHER_ERROR, error->message), NULL);
 	}
@@ -1211,8 +1211,9 @@ convert_contact_to_updatexml (ESoapMessage *msg,
 	id->id = e_contact_get (old_contact, E_CONTACT_UID);
 	id->change_key = e_contact_get (old_contact, E_CONTACT_REV);
 
-	e_ews_message_start_item_change (msg, E_EWS_ITEMCHANGE_TYPE_ITEM,
-					 id->id, id->change_key, 0);
+	e_ews_message_start_item_change (
+		msg, E_EWS_ITEMCHANGE_TYPE_ITEM,
+		id->id, id->change_key, 0);
 
 	/*Iterate for each field in contact*/
 
@@ -1222,7 +1223,7 @@ convert_contact_to_updatexml (ESoapMessage *msg,
 			value =  e_contact_get (new_contact, mappings[i].field_id);
 			old_value =  e_contact_get (old_contact, mappings[i].field_id);
 			if ((value && !old_value) || (!value && old_value) ||(value && old_value && g_ascii_strcasecmp (value, old_value)))
-				convert_contact_property_to_updatexml(msg, mappings[i].element_name, value, "contacts", NULL, NULL);
+				convert_contact_property_to_updatexml (msg, mappings[i].element_name, value, "contacts", NULL, NULL);
 			if (value)
 				g_free (value);
 			if (old_value)
@@ -1253,9 +1254,10 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 
 	if (vcards->next != NULL) {
 		e_data_book_respond_modify_contacts (book, opid,
-						     EDB_ERROR_EX (NOT_SUPPORTED,
-		                                     _("The backend does not support bulk modifications")),
-						     NULL);
+			EDB_ERROR_EX (
+			NOT_SUPPORTED,
+			_("The backend does not support bulk modifications")),
+			NULL);
 		return;
 	}
 
@@ -1298,8 +1300,9 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 		return;
 	}
 
-	old_contact = ews_book_backend_sqlitedb_get_contact ( priv->ebsdb, priv->folder_id,
-				 id->id, NULL, NULL, &error);
+	old_contact = ews_book_backend_sqlitedb_get_contact (
+		priv->ebsdb, priv->folder_id,
+		id->id, NULL, NULL, &error);
 	if (!old_contact) {
 		g_object_unref (contact);
 		e_data_book_respond_modify_contacts (book, opid, EDB_ERROR (NOT_SUPPORTED), NULL);
@@ -1421,7 +1424,7 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 		e_ews_connection_find_folder_items_sync (
 			priv->cnc, EWS_PRIORITY_MEDIUM,
 			fid, "IdOnly", NULL, NULL, query,
-			EWS_FOLDER_TYPE_CONTACTS,
+			E_EWS_FOLDER_TYPE_CONTACTS,
 			&includes_last_item,
 			&items, (EwsConvertQueryCallback) (e_ews_query_to_restriction),
 			cancellable, &error);
@@ -1431,7 +1434,7 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 		convert_error_to_edb_error (&error);
 		e_data_book_respond_get_contact_list (book, opid, error, vcard_list);
 
-		e_ews_folder_free_fid (fid);
+		e_ews_folder_id_free (fid);
 		g_slist_foreach (vcard_list, (GFunc) g_free, NULL);
 		g_slist_free (vcard_list);
 		return;
@@ -1621,9 +1624,10 @@ e_book_backend_ews_build_restriction (const gchar *query,
 	sdata->is_query_handled = TRUE;
 
 	for (i = 0; i < G_N_ELEMENTS (symbols); i++) {
-		e_sexp_add_function (sexp, 0, (gchar *) symbols[i].name,
-				     symbols[i].func,
-				     sdata);
+		e_sexp_add_function (
+			sexp, 0, (gchar *) symbols[i].name,
+			symbols[i].func,
+			sdata);
 	}
 
 	e_sexp_input_text (sexp, query, strlen (query));
@@ -1665,7 +1669,7 @@ ews_gal_needs_update (EBookBackendEws *cbews,
 	if (seq < full->seq)
 		ret = TRUE;
 
-	d(printf ("Gal needs update: %d \n", ret);)
+	d (printf ("Gal needs update: %d \n", ret);)
 exit:
 	g_free (tmp);
 	return ret;
@@ -1682,9 +1686,10 @@ ews_download_full_gal (EBookBackendEws *cbews,
 	gchar *full_url, *oab_url, *cache_file = NULL;
 	const gchar *cache_dir;
 	gchar *comp_cache_file = NULL, *uncompress_file = NULL;
-	CamelEwsSettings *settings;
+	gchar *password;
+	CamelEwsSettings *ews_settings;
 
-	settings = book_backend_ews_get_collection_settings (cbews);
+	ews_settings = book_backend_ews_get_collection_settings (cbews);
 
 	/* oab url with oab.xml removed from the suffix */
 	oab_url = g_strndup (priv->oab_url, strlen (priv->oab_url) - 7);
@@ -1692,10 +1697,12 @@ ews_download_full_gal (EBookBackendEws *cbews,
 	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (cbews));
 	comp_cache_file = g_build_filename (cache_dir, full->filename, NULL);
 
-	oab_cnc = e_ews_connection_new (full_url, priv->username, priv->password,
-		camel_network_settings_get_auth_mechanism (CAMEL_NETWORK_SETTINGS (settings)),
-		camel_ews_settings_get_timeout (settings),
-		NULL, NULL, NULL);
+	oab_cnc = e_ews_connection_new (full_url, ews_settings);
+
+	password = e_ews_connection_dup_password (priv->cnc);
+	e_ews_connection_set_password (oab_cnc, password);
+	g_free (password);
+
 	if (!e_ews_connection_download_oal_file_sync (
 		oab_cnc, comp_cache_file, NULL, NULL, cancellable, error))
 		goto exit;
@@ -1708,7 +1715,7 @@ ews_download_full_gal (EBookBackendEws *cbews,
 		goto exit;
 	}
 
-	d(g_print ("OAL file decompressed %s \n", uncompress_file);)
+	d (g_print ("OAL file decompressed %s \n", uncompress_file);)
 
 exit:
 	if (comp_cache_file)
@@ -1768,7 +1775,7 @@ ews_gal_store_contact (EContact *contact,
 		GSList *l;
 		gchar *status_message = NULL;
 
-		d(g_print ("GAL adding contacts, percent complete : %d \n", percent);)
+		d (g_print ("GAL adding contacts, percent complete : %d \n", percent);)
 
 		status_message = g_strdup_printf (_("Downloading contacts in %s %d%% completed... "), priv->folder_name, percent);
 		e_book_backend_foreach_view (E_BOOK_BACKEND (data->cbews), book_view_notify_status, status_message);
@@ -1843,18 +1850,20 @@ ebews_start_gal_sync (gpointer data)
 	GSList *full_l = NULL;
 	gboolean ret = TRUE;
 	gchar *uncompressed_filename = NULL;
-	CamelEwsSettings *settings;
+	gchar *password;
+	CamelEwsSettings *ews_settings;
 
 	cbews = (EBookBackendEws *) data;
-	settings = book_backend_ews_get_collection_settings (cbews);
+	ews_settings = book_backend_ews_get_collection_settings (cbews);
 	priv = cbews->priv;
 
-	oab_cnc = e_ews_connection_new (priv->oab_url, priv->username, priv->password,
-		camel_network_settings_get_auth_mechanism (CAMEL_NETWORK_SETTINGS (settings)),
-		camel_ews_settings_get_timeout (settings),
-		NULL, NULL, NULL);
+	oab_cnc = e_ews_connection_new (priv->oab_url, ews_settings);
 
-	d(printf ("Ewsgal: Fetching oal full details file \n");)
+	password = e_ews_connection_dup_password (priv->cnc);
+	e_ews_connection_set_password (oab_cnc, password);
+	g_free (password);
+
+	d (printf ("Ewsgal: Fetching oal full details file \n");)
 
 	if (!e_ews_connection_get_oal_detail_sync (
 		oab_cnc, priv->folder_id, "Full", &full_l,
@@ -1875,21 +1884,21 @@ ebews_start_gal_sync (gpointer data)
 	if (!ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
 		gchar *seq;
 
-		d(printf ("Ewsgal: Downloading full gal \n");)
+		d (printf ("Ewsgal: Downloading full gal \n");)
 		uncompressed_filename = ews_download_full_gal (cbews, full, priv->cancellable, &error);
 		if (error) {
 			ret = FALSE;
 			goto exit;
 		}
 
-		d(printf ("Ewsgal: Removing old gal \n");)
+		d (printf ("Ewsgal: Removing old gal \n");)
 		/* remove old_gal_file */
 		ret = ews_remove_old_gal_file (cbews, &error);
 		if (!ret) {
 			goto exit;
 		}
 
-		d(printf ("Ewsgal: Replacing old gal with new gal contents in db \n");)
+		d (printf ("Ewsgal: Replacing old gal with new gal contents in db \n");)
 		ret = ews_replace_gal_in_db (cbews, uncompressed_filename, priv->cancellable, &error);
 		if (!ret)
 			goto exit;
@@ -1904,7 +1913,7 @@ ebews_start_gal_sync (gpointer data)
 		}
 	}
 
-	d(printf ("Ews gal: sync successfull complete \n");)
+	d (printf ("Ews gal: sync successfull complete \n");)
 
 exit:
 	if (error) {
@@ -2081,7 +2090,7 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 			gint len = strlen (mb->name);
 			gchar *value;
 
-			if (mb->name [0] == '\"' && mb->name [len - 1] == '\"')
+			if (mb->name[0] == '\"' && mb->name[len - 1] == '\"')
 				value = g_strdup_printf ("%s <%s>", mb->name, mb->email);
 			else
 				value = g_strdup_printf ("\"%s\" <%s>", mb->name, mb->email);
@@ -2129,7 +2138,7 @@ ebews_vcards_append_dl (const EwsId *id,
 			gint len = strlen (mb->name);
 			gchar *value;
 
-			if (mb->name [0] == '\"' && mb->name [len - 1] == '\"')
+			if (mb->name[0] == '\"' && mb->name[len - 1] == '\"')
 				value = g_strdup_printf ("%s <%s>", mb->name, mb->email);
 			else
 				value = g_strdup_printf ("\"%s\" <%s>", mb->name, mb->email);
@@ -2330,6 +2339,9 @@ ebews_start_sync (gpointer data)
 
 	g_free (sync_state);
 
+	/* hide progress message when done */
+	e_book_backend_foreach_view (E_BOOK_BACKEND (ebews), book_view_notify_status, NULL);
+
 	if (error) {
 		g_warning ("Error Syncing Contacts: Folder %s Error: %s", priv->folder_id, error->message);
 		g_clear_error (&error);
@@ -2468,7 +2480,6 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 	GSList *ids = NULL, *mailboxes = NULL, *l, *contacts = NULL, *c;
 	EwsFolderId *fid;
 	ESource *source;
-	ESourceRegistry *registry;
 	ESourceEwsFolder *extension;
 	const gchar *extension_name;
 	GError *error = NULL;
@@ -2478,7 +2489,6 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 	priv = ebews->priv;
 	query = e_data_book_view_get_card_query (book_view);
 
-	registry = e_book_backend_get_registry (backend);
 	source = e_backend_get_source (E_BACKEND (backend));
 
 	e_data_book_view_ref (book_view);
@@ -2500,8 +2510,8 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 	if (priv->cnc == NULL) {
 		/* XXX Why doesn't start_book_view()
 		 *     get passed a GCancellable? */
-		e_source_registry_authenticate_sync (
-			registry, source,
+		e_backend_authenticate_sync (
+			E_BACKEND (backend),
 			E_SOURCE_AUTHENTICATOR (backend),
 			NULL, &error);
 		if (error != NULL) {
@@ -2553,7 +2563,7 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 		&includes_last_item, cancellable, &error);
 	g_free (auto_comp_str);
 	g_hash_table_remove (priv->ops, book_view);
-	e_ews_folder_free_fid (fid);
+	e_ews_folder_id_free (fid);
 	if (error != NULL) {
 		e_data_book_view_notify_complete (book_view, error);
 		e_data_book_view_unref (book_view);
@@ -2801,13 +2811,11 @@ e_book_backend_ews_open (EBookBackend *backend,
                          gboolean only_if_exists)
 {
 	EBookBackendEws *cbews;
-	ESourceRegistry *registry;
 	ESource *source;
 	GError *error = NULL;
 
 	cbews = E_BOOK_BACKEND_EWS (backend);
 
-	registry = e_book_backend_get_registry (backend);
 	source = e_backend_get_source (E_BACKEND (backend));
 	e_book_backend_ews_load_source (backend, source, only_if_exists, &error);
 
@@ -2821,8 +2829,8 @@ e_book_backend_ews_open (EBookBackend *backend,
 		PRIV_UNLOCK (cbews->priv);
 
 		if (need_to_authenticate)
-			e_source_registry_authenticate_sync (
-				registry, source,
+			e_backend_authenticate_sync (
+				E_BACKEND (backend),
 				E_SOURCE_AUTHENTICATOR (backend),
 				cancellable, &error);
 	}
@@ -2879,16 +2887,6 @@ e_book_backend_ews_dispose (GObject *object)
 		priv->folder_name = NULL;
 	}
 
-	if (priv->username) {
-		g_free (priv->username);
-		priv->username = NULL;
-	}
-
-	if (priv->password) {
-		g_free (priv->password);
-		priv->password = NULL;
-	}
-
 	if (priv->attachment_dir) {
 		g_free (priv->attachment_dir);
 		priv->attachment_dir = NULL;
@@ -2933,13 +2931,7 @@ book_backend_ews_try_password_sync (ESourceAuthenticator *authenticator,
 	EEwsConnection *connection;
 	ESourceAuthenticationResult result;
 	CamelEwsSettings *ews_settings;
-	CamelNetworkSettings *network_settings;
-	EwsFolderId *fid = NULL;
-	GSList *folders = NULL;
-	GSList *ids = NULL;
 	gchar *hosturl;
-	gchar *user;
-	GError *local_error = NULL;
 
 	/* This tests the password by fetching the contacts folder. */
 
@@ -2947,34 +2939,13 @@ book_backend_ews_try_password_sync (ESourceAuthenticator *authenticator,
 	ews_settings = book_backend_ews_get_collection_settings (backend);
 	hosturl = camel_ews_settings_dup_hosturl (ews_settings);
 
-	network_settings = CAMEL_NETWORK_SETTINGS (ews_settings);
-	user = camel_network_settings_dup_user (network_settings);
+	connection = e_ews_connection_new (hosturl, ews_settings);
 
-	connection = e_ews_connection_new (
-		hosturl, user, password->str,
-		camel_network_settings_get_auth_mechanism (network_settings),
-		camel_ews_settings_get_timeout (ews_settings),
-		NULL, NULL, error);
+	result = e_source_authenticator_try_password_sync (
+		E_SOURCE_AUTHENTICATOR (connection),
+		password, cancellable, error);
 
-	if (connection == NULL) {
-		result = E_SOURCE_AUTHENTICATION_ERROR;
-		goto exit;
-	}
-
-	fid = g_new0 (EwsFolderId, 1);
-	fid->id = g_strdup ("contacts");
-	fid->is_distinguished_id = TRUE;
-	ids = g_slist_append (ids, fid);
-
-	e_ews_connection_get_folder_sync (
-		connection, EWS_PRIORITY_MEDIUM, "Default",
-		NULL, ids, &folders, cancellable, &local_error);
-
-	e_ews_folder_free_fid (fid);
-	g_slist_free (ids);
-	ids = NULL;
-
-	if (local_error == NULL) {
+	if (result == E_SOURCE_AUTHENTICATION_ACCEPTED) {
 
 		PRIV_LOCK (backend->priv);
 
@@ -2982,38 +2953,12 @@ book_backend_ews_try_password_sync (ESourceAuthenticator *authenticator,
 			g_object_unref (backend->priv->cnc);
 		backend->priv->cnc = g_object_ref (connection);
 
-		/* Stash the username and password for later
-		 * reuse in Offline Address Book connections. */
-		g_free (backend->priv->username);
-		g_free (backend->priv->password);
-		backend->priv->username = g_strdup (user);
-		backend->priv->password = g_strdup (password->str);
-
 		PRIV_UNLOCK (backend->priv);
-
-		result = E_SOURCE_AUTHENTICATION_ACCEPTED;
-
-	} else {
-		gboolean auth_failed;
-
-		auth_failed = g_error_matches (
-			local_error, EWS_CONNECTION_ERROR,
-			EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED);
-
-		if (auth_failed) {
-			g_clear_error (&local_error);
-			result = E_SOURCE_AUTHENTICATION_REJECTED;
-		} else {
-			g_propagate_error (error, local_error);
-			result = E_SOURCE_AUTHENTICATION_ERROR;
-		}
 	}
 
 	g_object_unref (connection);
 
-exit:
 	g_free (hosturl);
-	g_free (user);
 
 	return result;
 }
