@@ -62,7 +62,8 @@
 struct _CamelEwsStorePrivate {
 	time_t last_refresh_time;
 	GMutex *get_finfo_lock;
-	EEwsConnection *cnc;
+	EEwsConnection *connection;
+	GMutex *connection_lock;
 };
 
 static gboolean	ews_store_construct	(CamelService *service, CamelSession *session,
@@ -213,83 +214,26 @@ ews_update_folder_hierarchy (CamelEwsStore *ews_store,
 	g_free (sync_state);
 }
 
-static void
-ews_store_update_cnc_timeout_cb (CamelEwsSettings *settings,
-				 GParamSpec *spec,
-				 CamelEwsStore *store)
-{
-	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
-	g_return_if_fail (CAMEL_IS_EWS_STORE (store));
-
-	if (store->priv->cnc)
-		e_ews_connection_set_timeout (store->priv->cnc,
-			camel_ews_settings_get_timeout (settings));
-}
-
-static void
-ews_store_authenticate (EEwsConnection *cnc,
-                        SoupMessage *msg,
-                        SoupAuth *auth,
-                        gboolean retrying,
-                        gpointer data)
-{
-	CamelNetworkSettings *network_settings;
-	CamelSettings *settings;
-	CamelService *service = data;
-	const gchar *password;
-	const gchar *user;
-
-	password = camel_service_get_password (service);
-	settings = camel_service_get_settings (service);
-
-	network_settings = CAMEL_NETWORK_SETTINGS (settings);
-	user = camel_network_settings_get_user (network_settings);
-
-	g_return_if_fail (password != NULL);
-
-	e_ews_connection_authenticate (cnc, auth, user, password, NULL);
-}
-
 static gboolean
 ews_connect_sync (CamelService *service,
                   GCancellable *cancellable,
                   GError **error)
 {
+	EEwsConnection *connection;
 	CamelEwsStore *ews_store;
-	CamelEwsStorePrivate *priv;
-	CamelEwsSettings *ews_settings;
-	CamelNetworkSettings *network_settings;
-	CamelSettings *settings;
 	CamelSession *session;
-	const gchar *hosturl;
-	const gchar *user;
 	gboolean success;
 
-	ews_store = (CamelEwsStore *) service;
-	priv = ews_store->priv;
+	ews_store = CAMEL_EWS_STORE (service);
 	session = camel_service_get_session (service);
-	settings = camel_service_get_settings (service);
-
-	network_settings = CAMEL_NETWORK_SETTINGS (settings);
-	user = camel_network_settings_get_user (network_settings);
-
-	ews_settings = CAMEL_EWS_SETTINGS (settings);
-	hosturl = camel_ews_settings_get_hosturl (ews_settings);
 
 	if (camel_service_get_connection_status (service) == CAMEL_SERVICE_DISCONNECTED)
 		return FALSE;
 
-	if (priv->cnc)
+	connection = camel_ews_store_ref_connection (ews_store);
+	if (connection != NULL) {
+		g_object_unref (connection);
 		return TRUE;
-
-	priv->cnc = e_ews_connection_new (hosturl, user, NULL,
-					  camel_network_settings_get_auth_mechanism (network_settings),
-					  camel_ews_settings_get_timeout (ews_settings),
-					  G_CALLBACK (ews_store_authenticate), service,
-					  error);
-
-	if (!priv->cnc) {
-		return FALSE;
 	}
 
 	/* Try running an operation that requires authentication
@@ -297,18 +241,12 @@ ews_connect_sync (CamelService *service,
 	success = camel_session_authenticate_sync (
 		session, service, NULL, cancellable, error);
 
-	if (!success) {
-		g_object_unref (priv->cnc);
-		priv->cnc = NULL;
-		return FALSE;
-	}
+	if (success)
+		camel_offline_store_set_online_sync (
+			CAMEL_OFFLINE_STORE (ews_store),
+			TRUE, cancellable, NULL);
 
-	g_signal_connect (ews_settings, "notify::timeout", G_CALLBACK (ews_store_update_cnc_timeout_cb), service);
-
-	camel_offline_store_set_online_sync (
-		CAMEL_OFFLINE_STORE (ews_store), TRUE, cancellable, NULL);
-
-	return TRUE;
+	return success;
 }
 
 static gboolean
@@ -320,14 +258,20 @@ ews_disconnect_sync (CamelService *service,
 	CamelEwsStore *ews_store = (CamelEwsStore *) service;
 	CamelServiceClass *service_class;
 
+	g_mutex_lock (ews_store->priv->connection_lock);
+
 	/* TODO cancel all operations in the connection */
-	if (ews_store->priv->cnc) {
-		g_signal_handlers_disconnect_by_data (camel_service_get_settings (service), service);
-		e_ews_connection_forget_password (ews_store->priv->cnc);
-		g_object_unref (ews_store->priv->cnc);
-		ews_store->priv->cnc = NULL;
+	if (ews_store->priv->connection != NULL) {
+		g_signal_handlers_disconnect_by_data (
+			camel_service_get_settings (service), service);
+		e_ews_connection_set_password (
+			ews_store->priv->connection, NULL);
+		g_object_unref (ews_store->priv->connection);
+		ews_store->priv->connection = NULL;
 	}
 
+	g_mutex_unlock (ews_store->priv->connection_lock);
+
 	service_class = CAMEL_SERVICE_CLASS (camel_ews_store_parent_class);
 	return service_class->disconnect_sync (service, clean, cancellable, error);
 }
@@ -343,7 +287,7 @@ static SystemFolder system_folder[] = {
 	{"deleteditems", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_TRASH},
 	{"drafts", CAMEL_FOLDER_SYSTEM},
 	{"inbox", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_INBOX},
-	{"journal", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_MEMOS},
+	{"journal", CAMEL_FOLDER_SYSTEM | CAMEL_EWS_FOLDER_TYPE_JOURNAL},
 	{"notes", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_MEMOS},
 	{"outbox", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_OUTBOX},
 	{"sentitems", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_SENT},
@@ -384,19 +328,42 @@ ews_authenticate_sync (CamelService *service,
 {
 	CamelAuthenticationResult result;
 	CamelEwsStore *ews_store;
-	CamelEwsStorePrivate *priv;
+	CamelSettings *settings;
+	CamelEwsSettings *ews_settings;
+	EEwsConnection *connection;
 	GSList *folders_created = NULL;
 	GSList *folders_updated = NULL;
 	GSList *folders_deleted = NULL;
 	GSList *folder_ids = NULL, *folders = NULL;
 	gboolean includes_last_folder = FALSE;
 	gboolean initial_setup = FALSE;
+	const gchar *password;
+	gchar *hosturl;
 	gchar *sync_state = NULL;
 	GError *local_error = NULL, *folder_err = NULL;
 	gint n = 0;
 
-	ews_store = (CamelEwsStore *) service;
-	priv = ews_store->priv;
+	ews_store = CAMEL_EWS_STORE (service);
+
+	password = camel_service_get_password (service);
+
+	if (password == NULL) {
+		g_set_error_literal (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			_("Authentication password not available"));
+		return CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	settings = camel_service_get_settings (service);
+
+	ews_settings = CAMEL_EWS_SETTINGS (settings);
+	hosturl = camel_ews_settings_dup_hosturl (ews_settings);
+
+	connection = e_ews_connection_new (hosturl, ews_settings);
+	e_ews_connection_set_password (connection, password);
+
+	g_free (hosturl);
 
 	/* XXX We need to run some operation that requires authentication
 	 *     but does not change any server-side state, so we can check
@@ -410,17 +377,30 @@ ews_authenticate_sync (CamelService *service,
 		initial_setup = TRUE;
 
 	e_ews_connection_sync_folder_hierarchy_sync (
-		priv->cnc, EWS_PRIORITY_MEDIUM,
+		connection, EWS_PRIORITY_MEDIUM,
 		&sync_state, &includes_last_folder,
 		&folders_created, &folders_updated, &folders_deleted,
 		cancellable, &local_error);
 
 	if (local_error == NULL) {
+		g_mutex_lock (ews_store->priv->connection_lock);
+		if (ews_store->priv->connection != NULL)
+			g_object_unref (ews_store->priv->connection);
+		ews_store->priv->connection = g_object_ref (connection);
+		g_mutex_unlock (ews_store->priv->connection_lock);
+
 		/* This consumes all allocated result data. */
 		ews_update_folder_hierarchy (
 			ews_store, sync_state, includes_last_folder,
 			folders_created, folders_deleted, folders_updated);
 	} else {
+		g_mutex_lock (ews_store->priv->connection_lock);
+		if (ews_store->priv->connection != NULL) {
+			g_object_unref (ews_store->priv->connection);
+			ews_store->priv->connection = NULL;
+		}
+		g_mutex_unlock (ews_store->priv->connection_lock);
+
 		g_free (sync_state);
 
 		/* Make sure we're not leaking anything. */
@@ -443,22 +423,22 @@ ews_authenticate_sync (CamelService *service,
 
 		/* fetch system folders first using getfolder operation*/
 		e_ews_connection_get_folder_sync (
-			ews_store->priv->cnc, EWS_PRIORITY_MEDIUM, "IdOnly",
+			connection, EWS_PRIORITY_MEDIUM, "IdOnly",
 			NULL, folder_ids, &folders,
 			cancellable, &folder_err);
 
 		if (g_slist_length (folders) && (g_slist_length (folders) != G_N_ELEMENTS (system_folder)))
-			d(printf("Error : not all folders are returned by getfolder operation"));
+			d (printf ("Error : not all folders are returned by getfolder operation"));
 		else if (folder_err == NULL && folders != NULL)
 			ews_store_set_flags (ews_store, folders);
 		else if (folder_err) {
 			/*report error and make sure we are not leaking anything*/
 			g_warn_if_fail (folders == NULL);
 		} else
-			d(printf ("folders for respective distinguished ids don't exist"));
+			d (printf ("folders for respective distinguished ids don't exist"));
 
 		g_slist_foreach (folders, (GFunc) g_object_unref, NULL);
-		g_slist_foreach (folder_ids, (GFunc) e_ews_folder_free_fid, NULL);
+		g_slist_foreach (folder_ids, (GFunc) e_ews_folder_id_free, NULL);
 		g_slist_free (folders);
 		g_slist_free (folder_ids);
 		g_clear_error (&folder_err);
@@ -468,14 +448,14 @@ ews_authenticate_sync (CamelService *service,
 		result = CAMEL_AUTHENTICATION_ACCEPTED;
 	} else if (g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED)) {
 		g_clear_error (&local_error);
-		e_ews_connection_forget_password (priv->cnc);
 		result = CAMEL_AUTHENTICATION_REJECTED;
 	} else {
 		g_propagate_error (error, local_error);
-		e_ews_connection_forget_password (priv->cnc);
 		result = CAMEL_AUTHENTICATION_ERROR;
 	}
 
+	g_object_unref (connection);
+
 	return result;
 }
 
@@ -533,9 +513,10 @@ ews_get_folder_sync (CamelStore *store,
 
 		camel_folder_info_free (fi);
 	} else if (!fid) {
-		g_set_error (error, CAMEL_STORE_ERROR,
-			     CAMEL_ERROR_GENERIC,
-			     _("No such folder: %s"), folder_name);
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_ERROR_GENERIC,
+			_("No such folder: %s"), folder_name);
 		return NULL;
 	} else {
 		/* We don't actually care what it is; only that it exists */
@@ -571,14 +552,14 @@ folder_info_from_store_summary (CamelEwsStore *store,
 
 	for (l = folders; l != NULL; l = g_slist_next (l)) {
 		CamelFolderInfo *fi;
-		gint64 ftype;
+		EEwsFolderType ftype;
 
 		ftype = camel_ews_store_summary_get_folder_type (ews_summary, l->data, NULL);
-		if (ftype != EWS_FOLDER_TYPE_MAILBOX)
+		if (ftype != E_EWS_FOLDER_TYPE_MAILBOX)
 			continue;
 
 		fi = camel_ews_utils_build_folder_info (store, l->data);
-		g_ptr_array_add	(folder_infos, fi);
+		g_ptr_array_add (folder_infos, fi);
 	}
 
 	root_fi = camel_folder_info_build (folder_infos, top, '/', TRUE);
@@ -604,9 +585,10 @@ ews_folder_hierarchy_ready_cb (GObject *obj,
 	gboolean includes_last_folder;
 	GError *error = NULL;
 
-	e_ews_connection_sync_folder_hierarchy_finish	(cnc, res, &sync_state, &includes_last_folder,
-							 &folders_created, &folders_updated,
-							 &folders_deleted, &error);
+	e_ews_connection_sync_folder_hierarchy_finish (
+		cnc, res, &sync_state, &includes_last_folder,
+		&folders_created, &folders_updated,
+		&folders_deleted, &error);
 
 	if (error != NULL) {
 		g_warning ("Unable to fetch the folder hierarchy: %s :%d \n", error->message, error->code);
@@ -618,8 +600,9 @@ ews_folder_hierarchy_ready_cb (GObject *obj,
 		g_mutex_unlock (priv->get_finfo_lock);
 		goto exit;
 	}
-	ews_update_folder_hierarchy (ews_store, sync_state, includes_last_folder,
-				     folders_created, folders_deleted, folders_updated);
+	ews_update_folder_hierarchy (
+		ews_store, sync_state, includes_last_folder,
+		folders_created, folders_deleted, folders_updated);
 
 	g_mutex_lock (priv->get_finfo_lock);
 	ews_store->priv->last_refresh_time = time (NULL);
@@ -633,6 +616,7 @@ exit:
 static gboolean
 ews_refresh_finfo (CamelEwsStore *ews_store)
 {
+	EEwsConnection *connection;
 	gchar *sync_state;
 
 	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (ews_store)))
@@ -643,11 +627,17 @@ ews_refresh_finfo (CamelEwsStore *ews_store)
 
 	sync_state = camel_ews_store_summary_get_string_val (ews_store->summary, "sync_state", NULL);
 
+	connection = camel_ews_store_ref_connection (ews_store);
+
 	e_ews_connection_sync_folder_hierarchy (
-		ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
+		connection, EWS_PRIORITY_MEDIUM,
 		sync_state, NULL, ews_folder_hierarchy_ready_cb,
 		g_object_ref (ews_store));
+
+	g_object_unref (connection);
+
 	g_free (sync_state);
+
 	return TRUE;
 }
 
@@ -661,11 +651,13 @@ ews_get_folder_info_sync (CamelStore *store,
 	CamelEwsStore *ews_store;
 	CamelEwsStorePrivate *priv;
 	CamelFolderInfo *fi = NULL;
+	EEwsConnection *connection;
 	gchar *sync_state;
 	gboolean initial_setup = FALSE;
 	GSList *folders_created = NULL, *folders_updated = NULL;
 	GSList *folders_deleted = NULL;
 	gboolean includes_last_folder;
+	gboolean success;
 	GError *local_error = NULL;
 
 	ews_store = (CamelEwsStore *) store;
@@ -693,14 +685,21 @@ ews_get_folder_info_sync (CamelStore *store,
 		goto offline;
 	}
 
-	if (!e_ews_connection_sync_folder_hierarchy_sync (
-			ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
-			&sync_state, &includes_last_folder,
-			&folders_created, &folders_updated,
-			&folders_deleted, cancellable, &local_error)) {
+	connection = camel_ews_store_ref_connection (ews_store);
+
+	success = e_ews_connection_sync_folder_hierarchy_sync (
+		connection, EWS_PRIORITY_MEDIUM,
+		&sync_state, &includes_last_folder,
+		&folders_created, &folders_updated,
+		&folders_deleted, cancellable, &local_error);
+
+	g_object_unref (connection);
+
+	if (!success) {
 		if (local_error)
-			g_warning ("Unable to fetch the folder hierarchy: %s :%d \n",
-				   local_error->message, local_error->code);
+			g_warning (
+				"Unable to fetch the folder hierarchy: %s :%d \n",
+				local_error->message, local_error->code);
 		else
 			g_warning ("Unable to fetch the folder hierarchy.\n");
 
@@ -710,8 +709,9 @@ ews_get_folder_info_sync (CamelStore *store,
 		g_mutex_unlock (priv->get_finfo_lock);
 		return NULL;
 	}
-	ews_update_folder_hierarchy (ews_store, sync_state, includes_last_folder,
-				     folders_created, folders_deleted, folders_updated);
+	ews_update_folder_hierarchy (
+		ews_store, sync_state, includes_last_folder,
+		folders_created, folders_deleted, folders_updated);
 	g_mutex_unlock (priv->get_finfo_lock);
 
 offline:
@@ -731,16 +731,20 @@ ews_create_folder_sync (CamelStore *store,
 	gchar *fid = NULL;
 	gchar *full_name;
 	EwsFolderId *folder_id;
+	EEwsConnection *connection;
 	CamelFolderInfo *fi = NULL;
+	gboolean success;
 	GError *local_error = NULL;
 
 	/* Get Parent folder ID */
 	if (parent_name && parent_name[0]) {
-		fid = camel_ews_store_summary_get_folder_id_from_name (ews_summary,
-								       parent_name);
+		fid = camel_ews_store_summary_get_folder_id_from_name (
+			ews_summary, parent_name);
 		if (!fid) {
-			g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-				     _("Parent folder %s does not exist"), parent_name);
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Parent folder %s does not exist"),
+				parent_name);
 			return NULL;
 		}
 	}
@@ -750,12 +754,17 @@ ews_create_folder_sync (CamelStore *store,
 		return NULL;
 	}
 
-	/* Make the call */
-	if (!e_ews_connection_create_folder_sync (
-			ews_store->priv->cnc,
-			EWS_PRIORITY_MEDIUM, fid,
-			FALSE, folder_name, &folder_id,
-			cancellable, &local_error)) {
+	connection = camel_ews_store_ref_connection (ews_store);
+
+	success = e_ews_connection_create_folder_sync (
+		connection,
+		EWS_PRIORITY_MEDIUM, fid,
+		FALSE, folder_name, E_EWS_FOLDER_TYPE_MAILBOX,
+		&folder_id, cancellable, &local_error);
+
+	g_object_unref (connection);
+
+	if (!success) {
 		camel_ews_store_maybe_disconnect (ews_store, local_error);
 		g_propagate_error (error, local_error);
 		g_free (fid);
@@ -768,13 +777,14 @@ ews_create_folder_sync (CamelStore *store,
 	else
 		full_name = g_strdup (folder_name);
 
-	camel_ews_store_summary_new_folder (ews_summary, folder_id->id,
-					    fid, folder_id->change_key,
-					    folder_name,
-					    EWS_FOLDER_TYPE_MAILBOX,
-					    0, 0);
+	camel_ews_store_summary_new_folder (
+		ews_summary, folder_id->id,
+		fid, folder_id->change_key,
+		folder_name,
+		E_EWS_FOLDER_TYPE_MAILBOX,
+		0, 0);
 	fi = camel_ews_utils_build_folder_info (ews_store, folder_id->id);
-	e_ews_folder_free_fid (folder_id);
+	e_ews_folder_id_free (folder_id);
 
 	camel_store_folder_created (store, fi);
 
@@ -793,13 +803,16 @@ ews_delete_folder_sync (CamelStore *store,
 	CamelEwsStoreSummary *ews_summary = ews_store->summary;
 	gchar *fid;
 	CamelFolderInfo *fi = NULL;
+	EEwsConnection *connection;
+	gboolean success;
 	GError *local_error = NULL;
 
-	fid = camel_ews_store_summary_get_folder_id_from_name (ews_summary,
-							       folder_name);
+	fid = camel_ews_store_summary_get_folder_id_from_name (
+		ews_summary, folder_name);
 	if (!fid) {
-		g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-			     _("Folder does not exist"));
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			_("Folder does not exist"));
 		return FALSE;
 	}
 
@@ -808,11 +821,17 @@ ews_delete_folder_sync (CamelStore *store,
 		return FALSE;
 	}
 
-	if (!e_ews_connection_delete_folder_sync (
-			ews_store->priv->cnc,
-			EWS_PRIORITY_MEDIUM,
-			fid, FALSE, "HardDelete",
-			cancellable, &local_error)) {
+	connection = camel_ews_store_ref_connection (ews_store);
+
+	success = e_ews_connection_delete_folder_sync (
+		connection,
+		EWS_PRIORITY_MEDIUM,
+		fid, FALSE, "HardDelete",
+		cancellable, &local_error);
+
+	g_object_unref (connection);
+
+	if (!success) {
 		camel_ews_store_maybe_disconnect (ews_store, local_error);
 		g_propagate_error (error, local_error);
 		g_free (fid);
@@ -842,11 +861,13 @@ rename_folder_cb (ESoapMessage *msg,
 {
 	struct _rename_cb_data *rename_data = user_data;
 
-	e_ews_message_start_item_change (msg, E_EWS_ITEMCHANGE_TYPE_FOLDER,
-					 rename_data->folder_id, rename_data->change_key, 0);
+	e_ews_message_start_item_change (
+		msg, E_EWS_ITEMCHANGE_TYPE_FOLDER,
+		rename_data->folder_id, rename_data->change_key, 0);
 	e_soap_message_start_element (msg, "SetFolderField", NULL, NULL);
-	e_ews_message_write_string_parameter_with_attribute (msg, "FieldURI", NULL, NULL,
-							      "FieldURI", "folder:DisplayName");
+	e_ews_message_write_string_parameter_with_attribute (
+		msg, "FieldURI", NULL, NULL,
+		"FieldURI", "folder:DisplayName");
 
 	e_soap_message_start_element (msg, "Folder", NULL, NULL);
 	e_ews_message_write_string_parameter (msg, "DisplayName", NULL, rename_data->display_name);
@@ -866,6 +887,7 @@ ews_rename_folder_sync (CamelStore *store,
 {
 	CamelEwsStore *ews_store = CAMEL_EWS_STORE (store);
 	CamelEwsStoreSummary *ews_summary = ews_store->summary;
+	EEwsConnection *connection;
 	const gchar *old_slash, *new_slash;
 	gchar *fid;
 	gchar *changekey;
@@ -881,21 +903,25 @@ ews_rename_folder_sync (CamelStore *store,
 
 	fid = camel_ews_store_summary_get_folder_id_from_name (ews_summary, old_name);
 	if (!fid) {
-		g_set_error (error, CAMEL_STORE_ERROR,
-			     CAMEL_STORE_ERROR_NO_FOLDER,
-			     _("Folder %s does not exist"), old_name);
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_STORE_ERROR_NO_FOLDER,
+			_("Folder %s does not exist"), old_name);
 		return FALSE;
 	}
 
 	changekey = camel_ews_store_summary_get_change_key (ews_summary, fid, error);
 	if (!changekey) {
 		g_free (fid);
-		g_set_error (error, CAMEL_STORE_ERROR,
-			     CAMEL_STORE_ERROR_NO_FOLDER,
-			     _("No change key record for folder %s"), fid);
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_STORE_ERROR_NO_FOLDER,
+			_("No change key record for folder %s"), fid);
 		return FALSE;
 	}
 
+	connection = camel_ews_store_ref_connection (ews_store);
+
 	old_slash = g_strrstr (old_name, "/");
 	new_slash = g_strrstr (new_name, "/");
 
@@ -929,9 +955,10 @@ ews_rename_folder_sync (CamelStore *store,
 		*/
 		if (new_slash - new_name != parent_len ||
 		    strncmp (old_name, new_name, parent_len)) {
-			g_set_error (error, CAMEL_STORE_ERROR,
-				     CAMEL_STORE_ERROR_INVALID,
-				     _("Cannot both rename and move a folder at the same time"));
+			g_set_error (
+				error, CAMEL_STORE_ERROR,
+				CAMEL_STORE_ERROR_INVALID,
+				_("Cannot both rename and move a folder at the same time"));
 			g_free (changekey);
 			goto out;
 		}
@@ -941,9 +968,12 @@ ews_rename_folder_sync (CamelStore *store,
 		rename_data->folder_id = fid;
 		rename_data->change_key = changekey;
 
-		if (!e_ews_connection_update_folder_sync (
-				ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
-				rename_folder_cb, rename_data, cancellable, &local_error)) {
+		res = e_ews_connection_update_folder_sync (
+			connection, EWS_PRIORITY_MEDIUM,
+			rename_folder_cb, rename_data,
+			cancellable, &local_error);
+
+		if (!res) {
 			g_free (rename_data);
 			goto out;
 		}
@@ -957,20 +987,24 @@ ews_rename_folder_sync (CamelStore *store,
 		 * the new parent folder */
 		if (new_slash != new_name) {
 			parent_name = g_strndup (new_name, new_slash - new_name - 1);
-			pfid = camel_ews_store_summary_get_folder_id_from_name (ews_summary,
-										parent_name);
+			pfid = camel_ews_store_summary_get_folder_id_from_name (
+				ews_summary, parent_name);
 			g_free (parent_name);
 			if (!pfid) {
-				g_set_error (error, CAMEL_STORE_ERROR,
-					     CAMEL_STORE_ERROR_NO_FOLDER,
-					     _("Cannot find folder ID for parent folder %s"),
-					     parent_name);
+				g_set_error (
+					error, CAMEL_STORE_ERROR,
+					CAMEL_STORE_ERROR_NO_FOLDER,
+					_("Cannot find folder ID for parent folder %s"),
+					parent_name);
 				goto out;
 			}
 		}
-		if (!e_ews_connection_move_folder_sync (
-				ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
-				pfid, fid, cancellable, &local_error)) {
+
+		res = e_ews_connection_move_folder_sync (
+			connection, EWS_PRIORITY_MEDIUM,
+			pfid, fid, cancellable, &local_error);
+
+		if (!res) {
 			g_free (pfid);
 			goto out;
 		}
@@ -985,6 +1019,8 @@ ews_rename_folder_sync (CamelStore *store,
 		g_propagate_error (error, local_error);
 	}
 
+	g_object_unref (connection);
+
 	g_free (changekey);
 	g_free (fid);
 	return res;
@@ -1006,16 +1042,28 @@ ews_get_name (CamelService *service,
 	user = camel_network_settings_get_user (network_settings);
 
 	if (brief)
-		return g_strdup_printf(_("Exchange server %s"), host);
+		return g_strdup_printf (
+			_("Exchange server %s"), host);
 	else
-		return g_strdup_printf(_("Exchange service for %s on %s"),
-				       user, host);
+		return g_strdup_printf (
+			_("Exchange service for %s on %s"), user, host);
 }
 
 EEwsConnection *
-camel_ews_store_get_connection (CamelEwsStore *ews_store)
+camel_ews_store_ref_connection (CamelEwsStore *ews_store)
 {
-	return g_object_ref (ews_store->priv->cnc);
+	EEwsConnection *connection = NULL;
+
+	g_return_val_if_fail (CAMEL_IS_EWS_STORE (ews_store), NULL);
+
+	g_mutex_lock (ews_store->priv->connection_lock);
+
+	if (ews_store->priv->connection != NULL)
+		connection = g_object_ref (ews_store->priv->connection);
+
+	g_mutex_unlock (ews_store->priv->connection_lock);
+
+	return connection;
 }
 
 static CamelFolder *
@@ -1096,7 +1144,7 @@ camel_ews_store_connected (CamelEwsStore *ews_store,
 
 void
 camel_ews_store_maybe_disconnect (CamelEwsStore *store,
-				  const GError *error)
+                                  const GError *error)
 {
 	CamelService *service;
 
@@ -1128,9 +1176,9 @@ ews_store_dispose (GObject *object)
 		ews_store->summary = NULL;
 	}
 
-	if (ews_store->priv->cnc != NULL) {
-		g_object_unref (ews_store->priv->cnc);
-		ews_store->priv->cnc = NULL;
+	if (ews_store->priv->connection != NULL) {
+		g_object_unref (ews_store->priv->connection);
+		ews_store->priv->connection = NULL;
 	}
 
 	/* Chain up to parent's dispose() method. */
@@ -1146,6 +1194,7 @@ ews_store_finalize (GObject *object)
 
 	g_free (ews_store->storage_path);
 	g_mutex_free (ews_store->priv->get_finfo_lock);
+	g_mutex_free (ews_store->priv->connection_lock);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_ews_store_parent_class)->finalize (object);
@@ -1191,7 +1240,7 @@ camel_ews_store_init (CamelEwsStore *ews_store)
 	ews_store->priv =
 		CAMEL_EWS_STORE_GET_PRIVATE (ews_store);
 
-	ews_store->priv->cnc = NULL;
 	ews_store->priv->last_refresh_time = time (NULL) - (FINFO_REFRESH_INTERVAL + 10);
 	ews_store->priv->get_finfo_lock = g_mutex_new ();
+	ews_store->priv->connection_lock = g_mutex_new ();
 }
