@@ -21,7 +21,6 @@
 #include <config.h>
 #include <glib/gi18n-lib.h>
 
-#include "server/e-ews-connection.h"
 #include "server/e-source-ews-folder.h"
 
 #define E_EWS_BACKEND_GET_PRIVATE(obj) \
@@ -33,12 +32,16 @@ typedef struct _SyncFoldersClosure SyncFoldersClosure;
 struct _EEwsBackendPrivate {
 	/* Folder ID -> ESource */
 	GHashTable *folders;
+	GMutex *folders_lock;
 
 	ESource *gal_source;
 	gchar *oal_selected;
 
 	gchar *sync_state;
 	GMutex *sync_state_lock;
+
+	EEwsConnection *connection;
+	GMutex *connection_lock;
 };
 
 struct _SyncFoldersClosure {
@@ -48,18 +51,10 @@ struct _SyncFoldersClosure {
 	GSList *folders_updated;
 };
 
-/* Forward Declarations */
-static void	e_ews_backend_authenticator_init
-				(ESourceAuthenticatorInterface *interface);
-
-G_DEFINE_DYNAMIC_TYPE_EXTENDED (
+G_DEFINE_DYNAMIC_TYPE (
 	EEwsBackend,
 	e_ews_backend,
-	E_TYPE_COLLECTION_BACKEND,
-	0,
-	G_IMPLEMENT_INTERFACE_DYNAMIC (
-		E_TYPE_SOURCE_AUTHENTICATOR,
-		e_ews_backend_authenticator_init))
+	E_TYPE_COLLECTION_BACKEND)
 
 static void
 sync_folders_closure_free (SyncFoldersClosure *closure)
@@ -84,6 +79,78 @@ sync_folders_closure_free (SyncFoldersClosure *closure)
 	g_slice_free (SyncFoldersClosure, closure);
 }
 
+static gboolean
+ews_backend_folders_contains (EEwsBackend *backend,
+                              const gchar *folder_id)
+{
+	gboolean contains;
+
+	g_return_val_if_fail (folder_id != NULL, FALSE);
+
+	g_mutex_lock (backend->priv->folders_lock);
+
+	contains = g_hash_table_contains (backend->priv->folders, folder_id);
+
+	g_mutex_unlock (backend->priv->folders_lock);
+
+	return contains;
+}
+
+static void
+ews_backend_folders_insert (EEwsBackend *backend,
+                            const gchar *folder_id,
+                            ESource *source)
+{
+	g_return_if_fail (folder_id != NULL);
+	g_return_if_fail (E_IS_SOURCE (source));
+
+	g_mutex_lock (backend->priv->folders_lock);
+
+	g_hash_table_insert (
+		backend->priv->folders,
+		g_strdup (folder_id),
+		g_object_ref (source));
+
+	g_mutex_unlock (backend->priv->folders_lock);
+}
+
+static ESource *
+ews_backend_folders_lookup (EEwsBackend *backend,
+                            const gchar *folder_id)
+{
+	ESource *source;
+
+	g_return_val_if_fail (folder_id != NULL, NULL);
+
+	g_mutex_lock (backend->priv->folders_lock);
+
+	source = g_hash_table_lookup (backend->priv->folders, folder_id);
+
+	if (source != NULL)
+		g_object_ref (source);
+
+	g_mutex_unlock (backend->priv->folders_lock);
+
+	return source;
+}
+
+static gboolean
+ews_backend_folders_remove (EEwsBackend *backend,
+                            const gchar *folder_id)
+{
+	gboolean removed;
+
+	g_return_val_if_fail (folder_id != NULL, FALSE);
+
+	g_mutex_lock (backend->priv->folders_lock);
+
+	removed = g_hash_table_remove (backend->priv->folders, folder_id);
+
+	g_mutex_unlock (backend->priv->folders_lock);
+
+	return removed;
+}
+
 static CamelEwsSettings *
 ews_backend_get_settings (EEwsBackend *backend)
 {
@@ -100,26 +167,6 @@ ews_backend_get_settings (EEwsBackend *backend)
 	return CAMEL_EWS_SETTINGS (settings);
 }
 
-static void
-ews_backend_queue_auth_session (ECollectionBackend *backend)
-{
-	ESourceRegistryServer *server;
-	EAuthenticationSession *session;
-	ESource *source;
-
-	server = e_collection_backend_ref_server (backend);
-	source = e_backend_get_source (E_BACKEND (backend));
-
-	session = e_authentication_session_new (
-		server, E_SOURCE_AUTHENTICATOR (backend),
-		e_source_get_uid (source));
-
-	e_source_registry_server_queue_auth_session (server, session);
-
-	g_object_unref (session);
-	g_object_unref (server);
-}
-
 static ESource *
 ews_backend_new_child (EEwsBackend *backend,
                        EEwsFolder *folder)
@@ -140,13 +187,13 @@ ews_backend_new_child (EEwsBackend *backend,
 	e_source_set_display_name (source, display_name);
 
 	switch (e_ews_folder_get_folder_type (folder)) {
-		case EWS_FOLDER_TYPE_CALENDAR:
+		case E_EWS_FOLDER_TYPE_CALENDAR:
 			extension_name = E_SOURCE_EXTENSION_CALENDAR;
 			break;
-		case EWS_FOLDER_TYPE_TASKS:
+		case E_EWS_FOLDER_TYPE_TASKS:
 			extension_name = E_SOURCE_EXTENSION_TASK_LIST;
 			break;
-		case EWS_FOLDER_TYPE_CONTACTS:
+		case E_EWS_FOLDER_TYPE_CONTACTS:
 			extension_name = E_SOURCE_EXTENSION_ADDRESS_BOOK;
 			break;
 		default:
@@ -169,6 +216,9 @@ ews_backend_new_child (EEwsBackend *backend,
 	e_source_offline_set_stay_synchronized (
 		E_SOURCE_OFFLINE (extension), TRUE);
 
+	e_server_side_source_set_remote_deletable (
+		E_SERVER_SIDE_SOURCE (source), TRUE);
+
 	return source;
 }
 
@@ -216,19 +266,19 @@ ews_backend_sync_created_folders (EEwsBackend *backend,
 		fid = e_ews_folder_get_id (folder);
 		if (fid->id == NULL)
 			continue;  /* not a valid ID anyway */
-		if (g_hash_table_contains (backend->priv->folders, fid->id))
+		if (ews_backend_folders_contains (backend, fid->id))
 			continue;
 
 		switch (e_ews_folder_get_folder_type (folder)) {
-			case EWS_FOLDER_TYPE_CALENDAR:
+			case E_EWS_FOLDER_TYPE_CALENDAR:
 				source = ews_backend_new_calendar (
 					backend, folder);
 				break;
-			case EWS_FOLDER_TYPE_TASKS:
+			case E_EWS_FOLDER_TYPE_TASKS:
 				source = ews_backend_new_task_list (
 					backend, folder);
 				break;
-			case EWS_FOLDER_TYPE_CONTACTS:
+			case E_EWS_FOLDER_TYPE_CONTACTS:
 				source = ews_backend_new_address_book (
 					backend, folder);
 				break;
@@ -261,8 +311,8 @@ ews_backend_sync_deleted_folders (EEwsBackend *backend,
 		ESource *source = NULL;
 
 		if (folder_id != NULL)
-			source = g_hash_table_lookup (
-				backend->priv->folders, folder_id);
+			source = ews_backend_folders_lookup (
+				backend, folder_id);
 
 		if (source == NULL)
 			continue;
@@ -270,6 +320,8 @@ ews_backend_sync_deleted_folders (EEwsBackend *backend,
 		/* This will trigger a "child-removed" signal and
 		 * our handler will remove the hash table entry. */
 		e_source_registry_server_remove_source (server, source);
+
+		g_object_unref (source);
 	}
 
 	g_object_unref (server);
@@ -326,7 +378,7 @@ ews_backend_add_gal_source (EEwsBackend *backend)
 
 	if (oal_selected == NULL) {
 		display_name = _("Global Address List");
-		oal_id = NULL;
+		oal_id = "global-address-list";
 	}
 
 	g_free (backend->priv->oal_selected);
@@ -426,6 +478,11 @@ ews_backend_dispose (GObject *object)
 
 	g_hash_table_remove_all (priv->folders);
 
+	if (priv->connection != NULL) {
+		g_object_unref (priv->connection);
+		priv->connection = NULL;
+	}
+
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (e_ews_backend_parent_class)->dispose (object);
 }
@@ -438,26 +495,45 @@ ews_backend_finalize (GObject *object)
 	priv = E_EWS_BACKEND_GET_PRIVATE (object);
 
 	g_hash_table_destroy (priv->folders);
+	g_mutex_free (priv->folders_lock);
 
 	g_free (priv->oal_selected);
 
 	g_free (priv->sync_state);
 	g_mutex_free (priv->sync_state_lock);
 
+	g_mutex_free (priv->connection_lock);
+
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_ews_backend_parent_class)->finalize (object);
 }
 
+static void
+ews_backend_constructed (GObject *object)
+{
+	ESource *source;
+
+	/* Chain up to parent's constructed() method. */
+	G_OBJECT_CLASS (e_ews_backend_parent_class)->constructed (object);
+
+	source = e_backend_get_source (E_BACKEND (object));
+
+	/* XXX Wondering if we ought to delay this until after folders
+	 *     are initially populated, just to remove the possibility
+	 *     of weird races with clients trying to create folders. */
+	e_server_side_source_set_remote_creatable (
+		E_SERVER_SIDE_SOURCE (source), TRUE);
+}
+
 static void
 ews_backend_populate (ECollectionBackend *backend)
 {
 	ESource *source;
 
-	/* We test authentication passwords by attempting to synchronize
-	 * the folder hierarchy.  Since we want to synchronize the folder
-	 * hierarchy immediately on startup, schedule an authentication
-	 * session first thing. */
-	ews_backend_queue_auth_session (backend);
+	/* For now at least, we don't need to know the
+	 * results, so no callback function is needed. */
+	e_ews_backend_sync_folders (
+		E_EWS_BACKEND (backend), NULL, NULL, NULL);
 
 	ews_backend_add_gal_source (E_EWS_BACKEND (backend));
 
@@ -485,13 +561,10 @@ static void
 ews_backend_child_added (ECollectionBackend *backend,
                          ESource *child_source)
 {
-	EEwsBackendPrivate *priv;
 	ESource *collection_source;
 	const gchar *extension_name;
 	gboolean is_mail = FALSE;
 
-	priv = E_EWS_BACKEND_GET_PRIVATE (backend);
-
 	collection_source = e_backend_get_source (E_BACKEND (backend));
 
 	extension_name = E_SOURCE_EXTENSION_MAIL_ACCOUNT;
@@ -539,10 +612,12 @@ ews_backend_child_added (ECollectionBackend *backend,
 		extension = e_source_get_extension (
 			child_source, extension_name);
 		folder_id = e_source_ews_folder_dup_id (extension);
-		if (folder_id != NULL)
-			g_hash_table_insert (
-				priv->folders, folder_id,
-				g_object_ref (child_source));
+		if (folder_id != NULL) {
+			ews_backend_folders_insert (
+				E_EWS_BACKEND (backend),
+				folder_id, child_source);
+			g_free (folder_id);
+		}
 	}
 
 	/* Chain up to parent's child_added() method. */
@@ -554,11 +629,8 @@ static void
 ews_backend_child_removed (ECollectionBackend *backend,
                            ESource *child_source)
 {
-	EEwsBackendPrivate *priv;
 	const gchar *extension_name;
 
-	priv = E_EWS_BACKEND_GET_PRIVATE (backend);
-
 	/* We track EWS folders in a hash table by folder ID. */
 	extension_name = E_SOURCE_EXTENSION_EWS_FOLDER;
 	if (e_source_has_extension (child_source, extension_name)) {
@@ -569,7 +641,8 @@ ews_backend_child_removed (ECollectionBackend *backend,
 			child_source, extension_name);
 		folder_id = e_source_ews_folder_get_id (extension);
 		if (folder_id != NULL)
-			g_hash_table_remove (priv->folders, folder_id);
+			ews_backend_folders_remove (
+				E_EWS_BACKEND (backend), folder_id);
 	}
 
 	/* Chain up to parent's child_removed() method. */
@@ -577,118 +650,163 @@ ews_backend_child_removed (ECollectionBackend *backend,
 		child_removed (backend, child_source);
 }
 
-static ESourceAuthenticationResult
-ews_backend_try_password_sync (ESourceAuthenticator *authenticator,
-                               const GString *password,
-                               GCancellable *cancellable,
-                               GError **error)
+static gboolean
+ews_backend_create_resource_sync (ECollectionBackend *backend,
+                                  ESource *source,
+                                  GCancellable *cancellable,
+                                  GError **error)
 {
-	EEwsBackend *backend;
 	EEwsConnection *connection;
-	ESource *source;
-	ESourceCollection *collection_extension;
-	ESourceAuthenticationResult result;
-	CamelEwsSettings *settings;
-	GSList *folders_created = NULL;
-	GSList *folders_updated = NULL;
-	GSList *folders_deleted = NULL;
-	gboolean includes_last_folder = FALSE;
+	EwsFolderId *out_folder_id = NULL;
+	EEwsFolderType folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
 	const gchar *extension_name;
-	gchar *sync_state;
-	gchar *hosturl;
-	gchar *user;
-	GError *local_error = NULL;
-
-	/* This tests the password by updating the folder hierarchy. */
-
-	backend = E_EWS_BACKEND (authenticator);
-	source = e_backend_get_source (E_BACKEND (backend));
-
-	settings = ews_backend_get_settings (backend);
-	hosturl = camel_ews_settings_dup_hosturl (settings);
-
-	extension_name = E_SOURCE_EXTENSION_COLLECTION;
-	collection_extension = e_source_get_extension (source, extension_name);
-	user = e_source_collection_dup_identity (collection_extension);
-
-	connection = e_ews_connection_new (
-		hosturl, user, password->str,
-		camel_network_settings_get_auth_mechanism (CAMEL_NETWORK_SETTINGS (settings)),
-		camel_ews_settings_get_timeout (settings),
-		NULL, NULL, error);
-
-	g_free (hosturl);
-	g_free (user);
+	const gchar *parent_folder_id = NULL;
+	gchar *folder_name;
+	gboolean success = FALSE;
 
+	connection = e_ews_backend_ref_connection_sync (
+		E_EWS_BACKEND (backend), cancellable, error);
 	if (connection == NULL)
-		return E_SOURCE_AUTHENTICATION_ERROR;
+		return FALSE;
 
-	g_mutex_lock (backend->priv->sync_state_lock);
-	sync_state = g_strdup (backend->priv->sync_state);
-	g_mutex_unlock (backend->priv->sync_state_lock);
-
-	/* XXX I think this leaks the old sync_state value when
-	 *     it replaces it with the new sync_state value. */
-	e_ews_connection_sync_folder_hierarchy_sync (
-		connection, EWS_PRIORITY_MEDIUM,
-		&sync_state, &includes_last_folder,
-		&folders_created, &folders_updated, &folders_deleted,
-		cancellable, &local_error);
-
-	g_object_unref (connection);
-
-	if (local_error == NULL) {
-		SyncFoldersClosure *closure;
-
-		/* We can now report the password was accepted.
-		 * Because a password dialog may be stuck in a busy
-		 * state, process the synchronization results from an
-		 * idle callback so we don't delay the authentication
-		 * session any longer than necessary. */
-
-		/* This takes ownership of the folder lists. */
-		closure = g_slice_new0 (SyncFoldersClosure);
-		closure->backend = g_object_ref (backend);
-		closure->folders_created = folders_created;
-		closure->folders_deleted = folders_deleted;
-		closure->folders_updated = folders_updated;
-
-		g_idle_add_full (
-			G_PRIORITY_DEFAULT_IDLE,
-			ews_backend_sync_folders_idle_cb, closure,
-			(GDestroyNotify) sync_folders_closure_free);
-
-		g_mutex_lock (backend->priv->sync_state_lock);
-		g_free (backend->priv->sync_state);
-		backend->priv->sync_state = sync_state;
-		g_mutex_unlock (backend->priv->sync_state_lock);
-
-		result = E_SOURCE_AUTHENTICATION_ACCEPTED;
-
-	} else {
-		gboolean auth_failed;
-
-		/* Make sure we're not leaking anything. */
-		g_warn_if_fail (folders_created == NULL);
-		g_warn_if_fail (folders_updated == NULL);
-		g_warn_if_fail (folders_deleted == NULL);
-
-		auth_failed = g_error_matches (
-			local_error, EWS_CONNECTION_ERROR,
-			EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED);
-
-		if (auth_failed) {
-			g_clear_error (&local_error);
-			result = E_SOURCE_AUTHENTICATION_REJECTED;
-		} else {
-			g_propagate_error (error, local_error);
-			result = E_SOURCE_AUTHENTICATION_ERROR;
-		}
-
-		g_free (sync_state);
+	extension_name = E_SOURCE_EXTENSION_ADDRESS_BOOK;
+	if (e_source_has_extension (source, extension_name)) {
+		folder_type = E_EWS_FOLDER_TYPE_CONTACTS;
+		parent_folder_id = "contacts";
 	}
 
-	return result;
+	extension_name = E_SOURCE_EXTENSION_CALENDAR;
+	if (e_source_has_extension (source, extension_name)) {
+		folder_type = E_EWS_FOLDER_TYPE_CALENDAR;
+		parent_folder_id = "calendar";
+	}
+
+	extension_name = E_SOURCE_EXTENSION_TASK_LIST;
+	if (e_source_has_extension (source, extension_name)) {
+		folder_type = E_EWS_FOLDER_TYPE_TASKS;
+		parent_folder_id = "tasks";
+	}
+
+	/* FIXME No support for memo lists. */
+
+	if (parent_folder_id == NULL) {
+		g_set_error (
+			error, G_IO_ERROR,
+			G_IO_ERROR_INVALID_ARGUMENT,
+			_("Could not determine a suitable folder "
+			"class for a new folder named '%s'"),
+			e_source_get_display_name (source));
+		goto exit;
+	}
+
+	folder_name = e_source_dup_display_name (source);
+
+	success = e_ews_connection_create_folder_sync (
+		connection, EWS_PRIORITY_MEDIUM,
+		parent_folder_id, TRUE,
+		folder_name, folder_type,
+		&out_folder_id, cancellable, error);
+
+	g_free (folder_name);
+
+	/* Sanity check */
+	g_warn_if_fail (
+		(success && out_folder_id != NULL) ||
+		(!success && out_folder_id == NULL));
+
+	if (out_folder_id != NULL) {
+		ESourceEwsFolder *extension;
+		const gchar *extension_name;
+
+		extension_name = E_SOURCE_EXTENSION_EWS_FOLDER;
+		extension = e_source_get_extension (source, extension_name);
+		e_source_ews_folder_set_id (
+			extension, out_folder_id->id);
+		e_source_ews_folder_set_change_key (
+			extension, out_folder_id->change_key);
+
+		e_ews_folder_id_free (out_folder_id);
+	}
+
+	if (success) {
+		ESourceRegistryServer *server;
+		ESource *parent_source;
+		const gchar *cache_dir;
+		const gchar *parent_uid;
+
+		/* Configure the source as a collection member. */
+		parent_source = e_backend_get_source (E_BACKEND (backend));
+		parent_uid = e_source_get_uid (parent_source);
+		e_source_set_parent (source, parent_uid);
+
+		/* Changes should be written back to the cache directory. */
+		cache_dir = e_collection_backend_get_cache_dir (backend);
+		e_server_side_source_set_write_directory (
+			E_SERVER_SIDE_SOURCE (source), cache_dir);
+
+		/* Set permissions for clients. */
+		e_server_side_source_set_writable (
+			E_SERVER_SIDE_SOURCE (source), TRUE);
+		e_server_side_source_set_remote_deletable (
+			E_SERVER_SIDE_SOURCE (source), TRUE);
+
+		server = e_collection_backend_ref_server (backend);
+		e_source_registry_server_add_source (server, source);
+		g_object_unref (server);
+	}
+
+exit:
+	g_object_unref (connection);
+
+	return success;
+}
+
+static gboolean
+ews_backend_delete_resource_sync (ECollectionBackend *backend,
+                                  ESource *source,
+                                  GCancellable *cancellable,
+                                  GError **error)
+{
+	EEwsConnection *connection;
+	ESourceEwsFolder *extension;
+	const gchar *extension_name;
+	gchar *folder_id;
+	gboolean success = FALSE;
+
+	connection = e_ews_backend_ref_connection_sync (
+		E_EWS_BACKEND (backend), cancellable, error);
+	if (connection == NULL)
+		return FALSE;
+
+	extension_name = E_SOURCE_EXTENSION_EWS_FOLDER;
+	if (!e_source_has_extension (source, extension_name)) {
+		g_set_error (
+			error, G_IO_ERROR,
+			G_IO_ERROR_INVALID_ARGUMENT,
+			_("Data source '%s' does not represent "
+			"an Exchange Web Services folder"),
+			e_source_get_display_name (source));
+		goto exit;
+	}
+	extension = e_source_get_extension (source, extension_name);
+	folder_id = e_source_ews_folder_dup_id (extension);
+
+	success = e_ews_connection_delete_folder_sync (
+		connection, EWS_PRIORITY_MEDIUM, folder_id,
+		FALSE, "HardDelete", cancellable, error);
+
+	if (success) {
+		ESourceRegistryServer *server;
+
+		server = e_collection_backend_ref_server (backend);
+		e_source_registry_server_remove_source (server, source);
+		g_object_unref (server);
+	}
+
+exit:
+	g_object_unref (connection);
+
+	return success;
 }
 
 static void
@@ -702,12 +820,15 @@ e_ews_backend_class_init (EEwsBackendClass *class)
 	object_class = G_OBJECT_CLASS (class);
 	object_class->dispose = ews_backend_dispose;
 	object_class->finalize = ews_backend_finalize;
+	object_class->constructed = ews_backend_constructed;
 
 	backend_class = E_COLLECTION_BACKEND_CLASS (class);
 	backend_class->populate = ews_backend_populate;
 	backend_class->dup_resource_id = ews_backend_dup_resource_id;
 	backend_class->child_added = ews_backend_child_added;
 	backend_class->child_removed = ews_backend_child_removed;
+	backend_class->create_resource_sync = ews_backend_create_resource_sync;
+	backend_class->delete_resource_sync = ews_backend_delete_resource_sync;
 
 	/* This generates an ESourceCamel subtype for CamelEwsSettings. */
 	e_source_camel_generate_subtype ("ews", CAMEL_TYPE_EWS_SETTINGS);
@@ -718,12 +839,6 @@ e_ews_backend_class_finalize (EEwsBackendClass *class)
 {
 }
 
-static void
-e_ews_backend_authenticator_init (ESourceAuthenticatorInterface *interface)
-{
-	interface->try_password_sync = ews_backend_try_password_sync;
-}
-
 static void
 e_ews_backend_init (EEwsBackend *backend)
 {
@@ -735,7 +850,9 @@ e_ews_backend_init (EEwsBackend *backend)
 		(GDestroyNotify) g_free,
 		(GDestroyNotify) g_object_unref);
 
+	backend->priv->folders_lock = g_mutex_new ();
 	backend->priv->sync_state_lock = g_mutex_new ();
+	backend->priv->connection_lock = g_mutex_new ();
 }
 
 void
@@ -747,3 +864,239 @@ e_ews_backend_type_register (GTypeModule *type_module)
 	e_ews_backend_register_type (type_module);
 }
 
+static void
+ews_backend_ref_connection_thread (GSimpleAsyncResult *simple,
+                                   GObject *object,
+                                   GCancellable *cancellable)
+{
+	EEwsConnection *connection;
+	GError *error = NULL;
+
+	connection = e_ews_backend_ref_connection_sync (
+		E_EWS_BACKEND (object), cancellable, &error);
+
+	/* Sanity check. */
+	g_return_if_fail (
+		((connection != NULL) && (error == NULL)) ||
+		((connection == NULL) && (error != NULL)));
+
+	if (connection != NULL)
+		g_simple_async_result_set_op_res_gpointer (
+			simple, connection, (GDestroyNotify) g_object_unref);
+
+	if (error != NULL)
+		g_simple_async_result_take_error (simple, error);
+}
+
+EEwsConnection *
+e_ews_backend_ref_connection_sync (EEwsBackend *backend,
+                                   GCancellable *cancellable,
+                                   GError **error)
+{
+	EEwsConnection *connection = NULL;
+	CamelEwsSettings *settings;
+	gchar *hosturl;
+	gboolean success;
+
+	g_return_val_if_fail (E_IS_EWS_BACKEND (backend), NULL);
+
+	g_mutex_lock (backend->priv->connection_lock);
+	if (backend->priv->connection != NULL)
+		connection = g_object_ref (backend->priv->connection);
+	g_mutex_unlock (backend->priv->connection_lock);
+
+	/* If we already have an authenticated
+	 * connection object, just return that. */
+	if (connection != NULL)
+		return connection;
+
+	settings = ews_backend_get_settings (backend);
+	hosturl = camel_ews_settings_dup_hosturl (settings);
+	connection = e_ews_connection_new (hosturl, settings);
+	g_free (hosturl);
+
+	success = e_backend_authenticate_sync (
+		E_BACKEND (backend),
+		E_SOURCE_AUTHENTICATOR (connection),
+		cancellable, error);
+
+	if (success) {
+		g_mutex_lock (backend->priv->connection_lock);
+		if (backend->priv->connection != NULL)
+			g_object_unref (backend->priv->connection);
+		backend->priv->connection = g_object_ref (connection);
+		g_mutex_unlock (backend->priv->connection_lock);
+	} else {
+		g_object_unref (connection);
+		connection = NULL;
+	}
+
+	return connection;
+}
+
+void
+e_ews_backend_ref_connection (EEwsBackend *backend,
+                              GCancellable *cancellable,
+                              GAsyncReadyCallback callback,
+                              gpointer user_data)
+{
+	GSimpleAsyncResult *simple;
+
+	g_return_if_fail (E_IS_EWS_BACKEND (backend));
+
+	simple = g_simple_async_result_new (
+		G_OBJECT (backend), callback,
+		user_data, e_ews_backend_ref_connection);
+
+	g_simple_async_result_run_in_thread (
+		simple, ews_backend_ref_connection_thread,
+		G_PRIORITY_DEFAULT, cancellable);
+
+	g_object_unref (simple);
+}
+
+EEwsConnection *
+e_ews_backend_ref_connection_finish (EEwsBackend *backend,
+                                     GAsyncResult *result,
+                                     GError **error)
+{
+	GSimpleAsyncResult *simple;
+	EEwsConnection *connection;
+
+	g_return_val_if_fail (
+		g_simple_async_result_is_valid (
+		result, G_OBJECT (backend),
+		e_ews_backend_ref_connection), NULL);
+
+	simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	if (g_simple_async_result_propagate_error (simple, error))
+		return NULL;
+
+	connection = g_simple_async_result_get_op_res_gpointer (simple);
+	g_return_val_if_fail (E_IS_EWS_CONNECTION (connection), NULL);
+
+	return g_object_ref (connection);
+}
+
+static void
+ews_backend_sync_folders_thread (GSimpleAsyncResult *simple,
+                                 GObject *object,
+                                 GCancellable *cancellable)
+{
+	GError *error = NULL;
+
+	e_ews_backend_sync_folders_sync (
+		E_EWS_BACKEND (object), cancellable, &error);
+
+	if (error != NULL)
+		g_simple_async_result_take_error (simple, error);
+}
+
+gboolean
+e_ews_backend_sync_folders_sync (EEwsBackend *backend,
+                                 GCancellable *cancellable,
+                                 GError **error)
+{
+	EEwsConnection *connection;
+	GSList *folders_created = NULL;
+	GSList *folders_updated = NULL;
+	GSList *folders_deleted = NULL;
+	gboolean includes_last_folder = FALSE;
+	gchar *sync_state;
+	gboolean success;
+
+	g_return_val_if_fail (E_IS_EWS_BACKEND (backend), FALSE);
+
+	connection = e_ews_backend_ref_connection_sync (
+		backend, cancellable, error);
+
+	if (connection == NULL)
+		return FALSE;
+
+	g_mutex_lock (backend->priv->sync_state_lock);
+	sync_state = g_strdup (backend->priv->sync_state);
+	g_mutex_unlock (backend->priv->sync_state_lock);
+
+	/* XXX I think this leaks the old sync_state value when
+	 *     it replaces it with the new sync_state value. */
+	success = e_ews_connection_sync_folder_hierarchy_sync (
+		connection, EWS_PRIORITY_MEDIUM,
+		&sync_state, &includes_last_folder,
+		&folders_created, &folders_updated, &folders_deleted,
+		cancellable, error);
+
+	if (success) {
+		SyncFoldersClosure *closure;
+
+		/* This takes ownership of the folder lists. */
+		closure = g_slice_new0 (SyncFoldersClosure);
+		closure->backend = g_object_ref (backend);
+		closure->folders_created = folders_created;
+		closure->folders_deleted = folders_deleted;
+		closure->folders_updated = folders_updated;
+
+		/* Process the results from an idle callback. */
+		g_idle_add_full (
+			G_PRIORITY_DEFAULT_IDLE,
+			ews_backend_sync_folders_idle_cb, closure,
+			(GDestroyNotify) sync_folders_closure_free);
+
+		g_mutex_lock (backend->priv->sync_state_lock);
+		g_free (backend->priv->sync_state);
+		backend->priv->sync_state = g_strdup (sync_state);
+		g_mutex_unlock (backend->priv->sync_state_lock);
+
+	} else {
+		/* Make sure we're not leaking anything. */
+		g_warn_if_fail (folders_created == NULL);
+		g_warn_if_fail (folders_updated == NULL);
+		g_warn_if_fail (folders_deleted == NULL);
+	}
+
+	g_free (sync_state);
+
+	g_object_unref (connection);
+
+	return success;
+}
+
+void
+e_ews_backend_sync_folders (EEwsBackend *backend,
+                            GCancellable *cancellable,
+                            GAsyncReadyCallback callback,
+                            gpointer user_data)
+{
+	GSimpleAsyncResult *simple;
+
+	g_return_if_fail (E_IS_EWS_BACKEND (backend));
+
+	simple = g_simple_async_result_new (
+		G_OBJECT (backend), callback,
+		user_data, e_ews_backend_sync_folders);
+
+	g_simple_async_result_run_in_thread (
+		simple, ews_backend_sync_folders_thread,
+		G_PRIORITY_DEFAULT, cancellable);
+
+	g_object_unref (simple);
+}
+
+gboolean
+e_ews_backend_sync_folders_finish (EEwsBackend *backend,
+                                   GAsyncResult *result,
+                                   GError **error)
+{
+	GSimpleAsyncResult *simple;
+
+	g_return_val_if_fail (
+		g_simple_async_result_is_valid (
+		result, G_OBJECT (backend),
+		e_ews_backend_sync_folders), FALSE);
+
+	simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	/* Assume success unless a GError is set. */
+	return !g_simple_async_result_propagate_error (simple, error);
+}
+
