@@ -119,28 +119,35 @@ typedef struct {
 	gchar *filename;
 } EwsOALDetails;
 
-typedef struct{
+typedef struct {
 	gchar *sid;
-	gchar *primary_smtp_add;
+	gchar *primary_smtp;
 	gchar *display_name;
 	gchar *distinguished_user;
 	gchar *external_user;
-}EwsUserId;
+} EwsUserId;
 
-typedef enum{
-	NONE,
-	EWS_PERM_EDITOR,
-	EWS_PERM_REVIEWER,
-	EWS_PERM_AUTHOR,
-	CUSTOM
-}EwsPermissionLevel;
+typedef enum {
+	EwsPermissionLevel_Unknown = 0,
+	EwsPermissionLevel_None,
+	EwsPermissionLevel_Reviewer,
+	EwsPermissionLevel_Author,
+	EwsPermissionLevel_Editor,
+	EwsPermissionLevel_Custom
+} EwsPermissionLevel;
 
-typedef struct{
+typedef struct {
 	EwsUserId *user_id;
-	EwsPermissionLevel calendar, contact, inbox, tasks, notes, journal;
+	EwsPermissionLevel calendar, tasks, inbox, contacts, notes, journal;
 	gboolean meetingcopies;
 	gboolean view_priv_items;
-}EwsDelegateInfo;
+} EwsDelegateInfo;
+
+typedef enum {
+	EwsDelegateDeliver_DelegatesOnly,
+	EwsDelegateDeliver_DelegatesAndMe,
+	EwsDelegateDeliver_DelegatesAndSendInformationToMe
+} EwsDelegateDeliver;
 
 typedef enum {
 	NORMAL_FIELD_URI,
@@ -174,41 +181,23 @@ typedef struct {
 	gpointer field_uri;
 } EwsSortOrder;
 
-typedef struct {
-	gchar *state;		/*Out of office state */
-	gchar *ext_aud;		/*external audience */
-	time_t start_tm;	/*Start time */
-	time_t end_tm;		/*End time */
-	gchar *int_reply;	/*Internal Reply */
-	gchar *ext_reply;	/*External Reply */
-} OOFSettings;
-
 void		ews_oal_free			(EwsOAL *oal);
 void		ews_oal_details_free		(EwsOALDetails *details);
 
 GType		e_ews_connection_get_type	(void);
 EEwsConnection *e_ews_connection_new		(const gchar *uri,
-						 const gchar *username,
-						 const gchar *password,
-						 const gchar *auth_mechanism,
-						 guint timeout,
-						 GCallback authenticate_cb,
-						 gpointer authenticate_ctx,
-						 GError **error);
+						 CamelEwsSettings *settings);
 const gchar *	e_ews_connection_get_uri	(EEwsConnection *cnc);
+const gchar *	e_ews_connection_get_password	(EEwsConnection *cnc);
+gchar *		e_ews_connection_dup_password	(EEwsConnection *cnc);
+void		e_ews_connection_set_password	(EEwsConnection *cnc,
+						 const gchar *password);
+CamelEwsSettings *
+		e_ews_connection_ref_settings	(EEwsConnection *cnc);
 SoupSession *	e_ews_connection_ref_soup_session
 						(EEwsConnection *cnc);
-void		e_ews_connection_forget_password
-						(EEwsConnection *cnc);
-void		e_ews_connection_set_timeout	(EEwsConnection *cnc,
-						 guint timeout);
 EEwsConnection *e_ews_connection_find		(const gchar *uri,
 						 const gchar *username);
-void		e_ews_connection_authenticate	(EEwsConnection *cnc,
-						 SoupAuth *auth,
-						 const gchar *user,
-						 const gchar *passwd,
-						 GError *error);
 void		e_ews_connection_queue_request	(EEwsConnection *cnc,
 						 ESoapMessage *msg,
 						 EEwsResponseCallback cb,
@@ -236,6 +225,7 @@ void		e_ews_connection_set_mailbox	(EEwsConnection *cnc,
 						 const gchar *email);
 
 void		ews_user_id_free		(EwsUserId *id);
+void		ews_delegate_info_free		(EwsDelegateInfo *info);
 
 void		e_ews_connection_sync_folder_items
 						(EEwsConnection *cnc,
@@ -274,7 +264,7 @@ gboolean	e_ews_connection_sync_folder_items_sync
 
 typedef void	(*EwsConvertQueryCallback)	(ESoapMessage *msg,
 						 const gchar *query,
-						 EwsFolderType type);
+						 EEwsFolderType type);
 
 void		e_ews_connection_find_folder_items
 						(EEwsConnection *cnc,
@@ -284,7 +274,7 @@ void		e_ews_connection_find_folder_items
 						 EwsAdditionalProps *add_props,
 						 EwsSortOrder *sort_order,
 						 const gchar *query,
-						 EwsFolderType type,
+						 EEwsFolderType type,
 						 EwsConvertQueryCallback convert_query_cb,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
@@ -303,7 +293,7 @@ gboolean	e_ews_connection_find_folder_items_sync
 						 EwsAdditionalProps *add_props,
 						 EwsSortOrder *sort_order,
 						 const gchar *query,
-						 EwsFolderType type,
+						 EEwsFolderType type,
 						 gboolean *includes_last_item,
 						 GSList **items,
 						 EwsConvertQueryCallback convert_query_cb,
@@ -530,6 +520,7 @@ void		e_ews_connection_create_folder	(EEwsConnection *cnc,
 						 const gchar *parent_folder_id,
 						 gboolean is_distinguished_id,
 						 const gchar *folder_name,
+						 EEwsFolderType folder_type,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
@@ -544,6 +535,7 @@ gboolean	e_ews_connection_create_folder_sync
 						 const gchar *parent_folder_id,
 						 gboolean is_distinguished_id,
 						 const gchar *folder_name,
+						 EEwsFolderType folder_type,
 						 EwsFolderId **folder_id,
 						 GCancellable *cancellable,
 						 GError **error);
@@ -799,62 +791,123 @@ gboolean	e_ews_connection_download_oal_file_finish
 void		e_ews_connection_get_delegate	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *mail_id,
-						 const gchar *include_permissions,
+						 gboolean include_permissions,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_get_delegate_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
-						 EwsDelegateInfo **get_delegate,
+						 EwsDelegateDeliver *deliver_to,
+						 GSList **delegates, /* EwsDelegateInfo * */
 						 GError **error);
 gboolean	e_ews_connection_get_delegate_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *mail_id,
-						 const gchar *include_permissions,
-						 EwsDelegateInfo **get_delegate,
+						 gboolean include_permissions,
+						 EwsDelegateDeliver *deliver_to,
+						 GSList **delegates, /* EwsDelegateInfo * */
 						 GCancellable *cancellable,
 						 GError **error);
-
-void		e_ews_connection_get_oof_settings
-						(EEwsConnection *cnc,
+void		e_ews_connection_add_delegate	(EEwsConnection *cnc,
 						 gint pri,
+						 const gchar *mail_id,
+						 const GSList *delegates, /* EwsDelegateInfo * */
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-gboolean	e_ews_connection_get_oof_settings_finish
+gboolean	e_ews_connection_add_delegate_finish
 						(EEwsConnection *cnc,
-						  GAsyncResult *result,
-						  OOFSettings **oof_settings,
-						  GError **error);
-gboolean	e_ews_connection_get_oof_settings_sync
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_add_delegate_sync
 						(EEwsConnection *cnc,
 						 gint pri,
-						 OOFSettings **oof_settings,
+						 const gchar *mail_id,
+						 const GSList *delegates, /* EwsDelegateInfo * */
 						 GCancellable *cancellable,
 						 GError **error);
-
-void		e_ews_connection_set_oof_settings
+void		e_ews_connection_remove_delegate
 						(EEwsConnection *cnc,
 						 gint pri,
-						 OOFSettings *oof_settings,
+						 const gchar *mail_id,
+						 const GSList *delegate_ids, /* EwsUserId * */
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-gboolean	e_ews_connection_set_oof_settings_finish
+gboolean	e_ews_connection_remove_delegate_finish
 						(EEwsConnection *cnc,
-						  GAsyncResult *result,
-						  GError **error);
-gboolean	e_ews_connection_set_oof_settings_sync
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_remove_delegate_sync
 						(EEwsConnection *cnc,
 						 gint pri,
-						 OOFSettings *oof_settings,
+						 const gchar *mail_id,
+						 const GSList *delegate_ids, /* EwsUserId * */
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_update_delegate
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 EwsDelegateDeliver deliver_to,
+						 const GSList *delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_update_delegate_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_update_delegate_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 EwsDelegateDeliver deliver_to,
+						 const GSList *delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_get_folder_permissions
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_folder_permissions_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GSList **permissions,
+						 GError **error);
+gboolean	e_ews_connection_get_folder_permissions_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 GSList **permissions,
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_set_folder_permissions
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 EEwsFolderType folder_type,
+						 const GSList *permissions,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_set_folder_permissions_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_set_folder_permissions_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 EEwsFolderType folder_type,
+						 const GSList *permissions,
 						 GCancellable *cancellable,
 						 GError **error);
-
-void		e_ews_connection_free_oof_settings
-						(OOFSettings *oof_settings);
 
 G_END_DECLS
 
