@@ -25,6 +25,7 @@
 #include <string.h>
 #include "e-ews-folder.h"
 #include "e-ews-message.h"
+#include "e-ews-enumtypes.h"
 
 G_DEFINE_TYPE (EEwsFolder, e_ews_folder, G_TYPE_OBJECT)
 
@@ -33,7 +34,7 @@ struct _EEwsFolderPrivate {
 	EwsFolderId *fid;
 	EwsFolderId *parent_fid;
 	gchar *folder_class;
-	EwsFolderType folder_type;
+	EEwsFolderType folder_type;
 	guint32 unread;
 	guint32 total;
 	guint32 child_count;
@@ -108,7 +109,7 @@ e_ews_folder_init (EEwsFolder *folder)
 	priv = g_new0 (EEwsFolderPrivate, 1);
 	folder->priv = priv;
 
-	priv->folder_type = EWS_FOLDER_TYPE_MAILBOX;
+	priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
 }
 
 /* FIXME pick it from folder_type and make it set folder_type */
@@ -139,13 +140,13 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 	g_return_val_if_fail (param != NULL, FALSE);
 
 	if ((node = e_soap_parameter_get_first_child_by_name (param, "Folder")))
-		priv->folder_type = EWS_FOLDER_TYPE_MAILBOX;
+		priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
 	else if ((node = e_soap_parameter_get_first_child_by_name (param, "CalendarFolder")))
-		priv->folder_type = EWS_FOLDER_TYPE_CALENDAR;
+		priv->folder_type = E_EWS_FOLDER_TYPE_CALENDAR;
 	else if ((node = e_soap_parameter_get_first_child_by_name (param, "ContactsFolder")))
-		priv->folder_type = EWS_FOLDER_TYPE_CONTACTS;
+		priv->folder_type = E_EWS_FOLDER_TYPE_CONTACTS;
 	else if ((node = e_soap_parameter_get_first_child_by_name (param, "TasksFolder")))
-		priv->folder_type = EWS_FOLDER_TYPE_TASKS;
+		priv->folder_type = E_EWS_FOLDER_TYPE_TASKS;
 	else {
 		g_warning ("Unable to find the Folder node \n");
 		return FALSE;
@@ -191,6 +192,54 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 	return TRUE;
 }
 
+const gchar *
+e_ews_folder_type_to_nick (EEwsFolderType folder_type)
+{
+	GEnumClass *enum_class;
+	GEnumValue *enum_value;
+	const gchar *folder_type_nick;
+
+	enum_class = g_type_class_ref (E_TYPE_EWS_FOLDER_TYPE);
+	enum_value = g_enum_get_value (enum_class, folder_type);
+
+	if (enum_value == NULL) {
+		folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
+		enum_value = g_enum_get_value (enum_class, folder_type);
+	}
+
+	g_return_val_if_fail (enum_value != NULL, NULL);
+
+	folder_type_nick = g_intern_string (enum_value->value_nick);
+
+	g_type_class_unref (enum_class);
+
+	return folder_type_nick;
+}
+
+EEwsFolderType
+e_ews_folder_type_from_nick (const gchar *folder_type_nick)
+{
+	GEnumClass *enum_class;
+	GEnumValue *enum_value;
+	EEwsFolderType folder_type;
+
+	g_return_val_if_fail (
+		folder_type_nick != NULL,
+		E_EWS_FOLDER_TYPE_UNKNOWN);
+
+	enum_class = g_type_class_ref (E_TYPE_EWS_FOLDER_TYPE);
+	enum_value = g_enum_get_value_by_nick (enum_class, folder_type_nick);
+
+	if (enum_value != NULL)
+		folder_type = enum_value->value;
+	else
+		folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
+
+	g_type_class_unref (enum_class);
+
+	return folder_type;
+}
+
 EEwsFolder *
 e_ews_folder_new_from_soap_parameter (ESoapParameter *param)
 {
@@ -207,8 +256,23 @@ e_ews_folder_new_from_soap_parameter (ESoapParameter *param)
 	return folder;
 }
 
+EwsFolderId *
+e_ews_folder_id_new (const gchar *id,
+                     const gchar *change_key,
+                     gboolean is_distinguished_id)
+{
+	EwsFolderId *fid;
+
+	fid = g_new0 (EwsFolderId, 1);
+	fid->id = g_strdup (id);
+	fid->change_key = g_strdup (change_key);
+	fid->is_distinguished_id = is_distinguished_id;
+
+	return fid;
+}
+
 void
-e_ews_folder_free_fid (EwsFolderId *fid)
+e_ews_folder_id_free (EwsFolderId *fid)
 {
 	if (fid) {
 		g_free (fid->id);
@@ -277,7 +341,7 @@ e_ews_folder_set_parent_id (EEwsFolder *folder,
 	priv->parent_fid = parent_fid;
 }
 
-EwsFolderType
+EEwsFolderType
 e_ews_folder_get_folder_type (EEwsFolder *folder)
 {
 	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), -1);
@@ -287,7 +351,7 @@ e_ews_folder_get_folder_type (EEwsFolder *folder)
 
 void
 e_ews_folder_set_folder_type (EEwsFolder *folder,
-                              EwsFolderType folder_type)
+                              EEwsFolderType folder_type)
 {
 	g_return_if_fail (E_IS_EWS_FOLDER (folder));
 
@@ -319,4 +383,3 @@ e_ews_folder_get_child_count (EEwsFolder *folder)
 	return folder->priv->child_count;
 
 }
-
