@@ -47,7 +47,7 @@ create_item_ready_callback (GObject *object,
 	GError *error = NULL;
 	GSList *ids = NULL, *l;
 
-	e_ews_connection_create_items_finish	(cnc, res, &ids, &error);
+	e_ews_connection_create_items_finish (cnc, res, &ids, &error);
 
 	if (error != NULL) {
 		g_print ("Unable to get item: %s :%d \n", error->message, error->code);
@@ -84,10 +84,10 @@ op_test_create_item_get_soap_msg (ESoapMessage *msg,
 
 	e_soap_message_end_element (msg);
 
-	g_printf("\nSOAP Request message:\n");
+	g_printf ("\nSOAP Request message:\n");
 	doc = e_soap_message_get_xml_doc (msg);
 	xmlDocDumpFormatMemory (doc, &xmlbuff,&size,1);
-	g_printf("%s\n", (gchar *)xmlbuff);
+	g_printf ("%s\n", (gchar *) xmlbuff);
 	xmlFree (xmlbuff);
 }
 
@@ -100,6 +100,7 @@ op_test_create_item ()
 	const gchar *folderid;
 	EEwsConnection *cnc;
 	GCancellable *cancellable;
+	CamelEwsSettings *settings;
 
 	cancellable = g_cancellable_new ();
 
@@ -108,8 +109,14 @@ op_test_create_item ()
 	g_assert_cmpstr (password, !=, NULL);
 	g_assert_cmpstr (uri, !=, NULL);
 
-	cnc = e_ews_connection_new (uri, username, password, NULL, 30, NULL, NULL, NULL);
-	g_assert (cnc != NULL);
+	settings = g_object_new (
+		CAMEL_TYPE_EWS_SETTINGS,
+		"user", username, NULL);
+
+	cnc = e_ews_connection_new (uri, settings);
+	e_ews_connection_set_password (cnc, password);
+
+	g_object_unref (settings);
 
 	//to check how to change to real data
 	folderid = "AQASAG1hbmR5Lnd1QGludGVsLmNvbQAuAAADnUl3sZrICU6Xd1qXV+rpVwEAJvRPgQpR30KhJ7wqBr4YXgAAAY6W+gAAAA==";
